/**
 * http://lejos.sourceforge.net/nxt/nxj/tutorial/MotorTutorial/ControllingMotors.htm
 */

import lejos.nxt.SensorPort;
import lejos.nxt.UltrasonicSensor;
import lejos.nxt.Motor;
import lejos.nxt.remote.RemoteMotor;
import lejos.util.Delay;

public class DVDChanger {

  public static UltrasonicSensor _ultraSonicSensor = new UltrasonicSensor(SensorPort.S1);
  public static RemoteMotor _diskStackMotor = Motor.A;
  public static RemoteMotor _diskInsertMotor = Motor.B;
  public static RemoteMotor _diskRemoveMotor = Motor.C;

  public static int _diskDistanceThreshold = 7;
  public static int _lightThreshold = 8;


  public static void removeDisk() {
    int removeCounter = 0;
    int MAXIMUM_REMOVE_COUNTER = 3;
    while (true) {
      System.out.println("Distance of disk to ultrasonic sensor: " + _ultraSonicSensor.getDistance());

      if (_ultraSonicSensor.getDistance() > _diskDistanceThreshold) {
        System.out.println("No disk in drawer.");
        return;
        // System.exit(4);
      }

      System.out.println("Disk found, trying to remove");

      int startAngle = 5;
      int intermediateAngle = 60;
      int maxAngle = 105;

      System.out.println("Angle of motor at the beginning: " + _diskRemoveMotor.getTachoCount());

      _diskRemoveMotor.setSpeed(80);
      _diskRemoveMotor.rotateTo(startAngle - intermediateAngle);
      System.out.println("Angle of motor after slow start: " + _diskRemoveMotor.getTachoCount());

      if (intermediateAngle != maxAngle) {
        Delay.msDelay(500);
        _diskRemoveMotor.setSpeed(360);
        _diskRemoveMotor.rotateTo(startAngle - maxAngle);
        System.out.println("Angle of motor after fast removal: " + _diskRemoveMotor.getTachoCount());
      }

      // rotate the motor into its initial position
      _diskRemoveMotor.rotateTo(startAngle);

      if (_ultraSonicSensor.getDistance() > _diskDistanceThreshold) {
        System.out.println("Disk removed successful !");
        return;
      } else {
        removeCounter++;
        if (removeCounter >= MAXIMUM_REMOVE_COUNTER) {
          System.out.println("Failed to remove disk.");
          System.exit(5);
        }
      }
    }
  }

  public static void insertDisk() {
    // check if a disk is still inserted, before inserting a new disk
    if (_ultraSonicSensor.getDistance() <= _diskDistanceThreshold) {
      System.err.println("Disk found. Please remove the disk first.");
      System.exit(2);
    }

    System.out.println("Trying to insert disk.");

    int counter = 0;
    int MAXIMUM_COUNTER = 7;

    while (counter < MAXIMUM_COUNTER) {
      _diskInsertMotor.setSpeed(360);
      _diskInsertMotor.rotate(-800, true);
      Delay.msDelay(200);

      _diskStackMotor.setSpeed(150);
      _diskStackMotor.rotate(-40);

      Delay.msDelay(2000);
      // check if disk was inserted successful
      if (_ultraSonicSensor.getDistance() <= _diskDistanceThreshold) {
        System.out.println("Disk inserted successfully.");
        break;
      }
      counter++;
    }

    if (counter == MAXIMUM_COUNTER) {
      System.out.println("No disk inserted !!!");
      System.exit(3);
    }

  }

  public static void usageInfo() {
    System.err.println("use at least one argument");
    System.err.println("insert or remove");
    System.exit(200);
  }

  /**
   * @param arguments
   */
  public static void main(String[] arguments) {
    if (arguments.length < 1) {
      usageInfo();
    }

    if (arguments[0].equals("insert")) {
      insertDisk();
    } else if (arguments[0].equals("remove")) {
      removeDisk();
    } else if (arguments[0].equals("removeInsert")) {
      removeDisk();
      insertDisk();
    } else {
      usageInfo();
    }

    System.out.println("Finished.");
    System.exit(0);
  }

}
